@@ -60,7 +60,7 @@ public:
       const PluginPath &path, const TranslatableString *pSymbol) override;
 
    bool IsPluginLoaded(const wxString& ID) const;
-   
+
    void RegisterPlugin(PluginDescriptor&& desc);
    const PluginID & RegisterPlugin(PluginProvider *provider) override;
    const PluginID & RegisterPlugin(PluginProvider *provider, ComponentInterface *command);
@@ -107,8 +107,9 @@ public:
 
    static PluginManager & Get();
 
-   static PluginID GetID(PluginProvider *provider);
-   static PluginID GetID(ComponentInterface *command);
+   
+   static PluginID GetID(const PluginProvider *provider);
+   static PluginID GetID(const ComponentInterface *command);
    static PluginID OldGetID(const EffectDefinitionInterface* effect);
    static PluginID GetID(const EffectDefinitionInterface* effect);
    //! Parse English effect name from the result of
@@ -183,13 +184,16 @@ public:
    void Load();
    //! Save to preferences
    void Save();
-   
+
    void NotifyPluginsChanged();
 
    //! What is the plugin registry version number now in the file?
    //! (Save() updates it)
    const PluginRegistryVersion &GetRegistryVersion() const override;
 
+   PluginPaths ReadCustomPaths(const PluginProvider& provider) override;
+   void StoreCustomPaths(const PluginProvider& provider, const PluginPaths& paths) override;
+
 private:
    // private! Use Get()
    PluginManager();
@@ -249,6 +253,6 @@ private:
 #define NYQUIST_PROMPT_NAME XO("Nyquist Prompt")
 
 // Latest version of the plugin registry config
-constexpr auto REGVERCUR = "1.3";
+constexpr auto REGVERCUR = "1.4";
 
 #endif /* __AUDACITY_PLUGINMANAGER_H__ */
