@@ -15,6 +15,7 @@
 
 #include "SelectedRegion.h"
 #include "Track.h"
+#include "FileNames.h"
 
 class wxTextFile;
 
@@ -25,6 +26,13 @@ class LabelTrack;
 struct LabelTrackHit;
 struct TrackPanelDrawingContext;
 
+enum class LabelFormat
+{
+   TEXT,
+   SUBRIP,
+   WEBVTT,
+};
+
 class AUDACITY_DLL_API LabelStruct
 {
 public:
@@ -43,9 +51,9 @@ public:
    void MoveLabel( int iEdge, double fNewTime);
 
    struct BadFormatException {};
-   static LabelStruct Import(wxTextFile &file, int &index);
+   static LabelStruct Import(wxTextFile &file, int &index, LabelFormat format);
 
-   void Export(wxTextFile &file) const;
+   void Export(wxTextFile &file, LabelFormat format, int index) const;
 
    /// Relationships between selection region and labels
    enum TimeRelations
@@ -116,18 +124,19 @@ class AUDACITY_DLL_API LabelTrack final
 
    using Holder = std::shared_ptr<LabelTrack>;
 
+   static const FileNames::FileType SubripFiles;
+   static const FileNames::FileType WebVTTFiles;
+
 private:
-   TrackListHolder Clone() const override;
-   void DoOnProjectTempoChange(
-      const std::optional<double>& oldTempo, double newTempo) override;
+   Track::Holder Clone(bool backup) const override;
 
 public:
    bool HandleXMLTag(const std::string_view& tag, const AttributesList& attrs) override;
    XMLTagHandler *HandleXMLChild(const std::string_view& tag) override;
    void WriteXML(XMLWriter &xmlFile) const override;
 
-   TrackListHolder Cut(double t0, double t1) override;
-   TrackListHolder Copy(double t0, double t1, bool forClipboard = true)
+   Track::Holder Cut(double t0, double t1) override;
+   Track::Holder Copy(double t0, double t1, bool forClipboard = true)
       const override;
    void Clear(double t0, double t1) override;
    void Paste(double t, const Track &src) override;
@@ -138,8 +147,9 @@ public:
    Silence(double t0, double t1, ProgressReporter reportProgress = {}) override;
    void InsertSilence(double t, double len) override;
 
-   void Import(wxTextFile & f);
-   void Export(wxTextFile & f) const;
+   static LabelFormat FormatForFileName(const wxString & fileName);
+   void Import(wxTextFile & f, LabelFormat format);
+   void Export(wxTextFile & f, LabelFormat format) const;
 
    int GetNumLabels() const;
    const LabelStruct *GetLabel(int index) const;
@@ -178,16 +188,21 @@ public:
       const override;
 
    struct Interval final : WideChannelGroupInterval {
-      Interval(const ChannelGroup &group,
-         double start, double end, size_t index
-      )  : WideChannelGroupInterval{ group, start, end }
+      Interval(const LabelTrack &track, size_t index
+      )  : mpTrack{ track.SharedPointer<const LabelTrack>() }
          , index{ index }
       {}
 
       ~Interval() override;
+      double Start() const override;
+      double End() const override;
+      size_t NChannels() const override;
       std::shared_ptr<ChannelInterval> DoGetChannel(size_t iChannel) override;
 
       size_t index;
+   private:
+      //! @invariant not null
+      const std::shared_ptr<const LabelTrack> mpTrack;
    };
    std::shared_ptr<Interval> MakeInterval(size_t index);
 
