@@ -15,10 +15,11 @@
 
 #include "ComponentInterface.h" // member variable
 
-#include "widgets/wxPanelWrapper.h" // to inherit
+#include "wxPanelWrapper.h" // to inherit
 
 class NumericTextCtrl;
 class ShuttleGui;
+class AudacityProject;
 
 class AUDACITY_DLL_API TimeDialog final : public wxDialogWrapper
 {
@@ -26,13 +27,12 @@ class AUDACITY_DLL_API TimeDialog final : public wxDialogWrapper
 
    TimeDialog(wxWindow *parent,
               const TranslatableString &title,
-              const NumericFormatSymbol &format,
-              double rate,
+              const NumericFormatID &format,
+              const AudacityProject &project,
               double time,
               const TranslatableString &prompt = XO("Duration"));
 
-   void SetFormatString(const NumericFormatSymbol &formatString);
-   void SetSampleRate(double sampleRate);
+   void SetFormatString(const NumericFormatID &formatString);
    void SetTimeValue(double newTime);
    const double GetTimeValue();
 
@@ -47,8 +47,8 @@ class AUDACITY_DLL_API TimeDialog final : public wxDialogWrapper
  private:
 
    TranslatableString mPrompt;
-   NumericFormatSymbol mFormat;
-   double mRate;
+   NumericFormatID mFormat;
+   const AudacityProject &mProject;
    double mTime;
 
    NumericTextCtrl *mTimeCtrl;
