@@ -39,7 +39,6 @@ ShuttleGui.
 #include "ProgressDialog.h"
 #include "HelpSystem.h"
 #include "AudacityMessageBox.h"
-#include "../widgets/VetoDialogHook.h"
 
 #include <unordered_map>
 
@@ -92,10 +91,6 @@ bool AudacityCommand::ShowInterface(wxWindow *parent, bool WXUNUSED(forceModal))
    mUIDialog->Fit();
    mUIDialog->SetMinSize(mUIDialog->GetSize());
 
-   // The Screenshot command might be popping this dialog up, just to capture it.
-   if ( VetoDialogHook::Call( mUIDialog ) )
-      return false;
-
    bool res = mUIDialog->ShowModal() != 0;
    return res;
 }
