@@ -11,7 +11,7 @@
 
 \file SetTrackCommand.cpp
 \brief Definitions for SetTrackCommand built up from 
-SetChannelsBase, SetTrackBase, SetTrackStatusCommand, SetTrackAudioCommand and
+SetTrackBase, SetTrackStatusCommand, SetTrackAudioCommand and
 SetTrackVisualsCommand
 
 \class SetTrackBase
@@ -20,17 +20,17 @@ loops over selected tracks. Subclasses override ApplyInner() to change
 one track.
 
 \class SetTrackStatusCommand
-\brief A SetChannelsBase that sets name, selected and focus.
+\brief A SetTrackBase that sets name, selected and focus.
 
 \class SetTrackAudioCommand
-\brief A SetChannelsBase that sets pan, gain, mute and solo.
+\brief A SetTrackBase that sets pan, gain, mute and solo.
 
 \class SetTrackVisualsCommand
-\brief A SetChannelsBase that sets appearance of a track.
+\brief A SetTrackBase that sets appearance of a track.
 
 \class SetTrackCommand
-\brief A SetChannelsBase that combines SetTrackStatusCommand,
-SetTrackAudioCommand and SetTrackVisualsCommand.
+\brief A SetTrackBase that combines SetTrackStatusCommand,
+SetTrackAudioConmmand and SetTrackVisualsCommand.
 
 *//*******************************************************************/
 
@@ -38,19 +38,21 @@ SetTrackAudioCommand and SetTrackVisualsCommand.
 #include "SetTrackInfoCommand.h"
 
 #include "CommandDispatch.h"
-#include "CommandManager.h"
+#include "MenuRegistry.h"
 #include "../CommonCommandFlags.h"
 #include "LoadCommands.h"
 #include "Project.h"
-#include "../TrackPanelAx.h"
+#include "TrackFocus.h"
 #include "../TrackPanel.h"
-#include "WaveTrack.h"
+#include "tracks/playabletrack/wavetrack/ui/WaveformAppearance.h"
 #include "../prefs/WaveformSettings.h"
+#include "WaveTrack.h"
 #include "../prefs/SpectrogramSettings.h"
 #include "SettingsVisitor.h"
 #include "ShuttleGui.h"
 #include "../tracks/playabletrack/wavetrack/ui/WaveChannelView.h"
 #include "../tracks/playabletrack/wavetrack/ui/WaveChannelViewConstants.h"
+#include "../tracks/playabletrack/wavetrack/ui/WaveformView.h"
 #include "CommandContext.h"
 
 bool SetTrackBase::Apply(const CommandContext & context)
@@ -326,18 +328,19 @@ bool SetTrackVisualsCommand::ApplyInner(
    const auto wt = dynamic_cast<WaveTrack *>(&t);
    if (!wt)
       return true;
+   auto &wc = **wt->Channels().begin();
    //auto pt = dynamic_cast<PlayableTrack *>(t);
    static const double ZOOMLIMIT = 0.001f;
 
    if (bHasColour)
-      wt->SetWaveColorIndex(mColour);
+      WaveformAppearance::Get(*wt).SetColorIndex(mColour);
 
    if (bHasHeight)
-      for (auto pChannel : t.Channels<WaveTrack>())
+      for (auto pChannel : t.Channels<WaveChannel>())
          ChannelView::Get(*pChannel).SetExpandedHeight(mHeight);
 
    if (bHasDisplayType) {
-      auto &view = WaveChannelView::Get(*wt);
+      auto &view = WaveChannelView::Get(wc);
       auto &all = WaveChannelSubViewType::All();
       if (mDisplayType < all.size())
          view.SetDisplay( all[ mDisplayType ].id );
@@ -397,9 +400,9 @@ bool SetTrackVisualsCommand::ApplyInner(
    if (bHasUseSpecPrefs) {
       if (bUseSpecPrefs)
          // reset it, and next we will be getting the defaults.
-         SpectrogramSettings::Reset(*wt);
+         SpectrogramSettings::Reset(wc);
       else
-         SpectrogramSettings::Own(*wt);
+         SpectrogramSettings::Own(wc);
    }
    auto &settings = SpectrogramSettings::Get(*wt);
    if (wt && bHasSpectralSelect)
@@ -423,12 +426,11 @@ bool SetTrackCommand::VisitSettings( ConstSettingsVisitor & S )
    { return VisitSettings<true>(S); }
 
 namespace {
-using namespace MenuTable;
+using namespace MenuRegistry;
 
 // Register menu items
 
 AttachedItem sAttachment1{
-   wxT("Optional/Extra/Part2/Scriptables1"),
    Items( wxT(""),
       // Note that the PLUGIN_SYMBOL must have a space between words,
       // whereas the short-form used here must not.
@@ -440,16 +442,17 @@ AttachedItem sAttachment1{
          CommandDispatch::OnAudacityCommand, AudioIONotBusyFlag() ),
       Command( wxT("SetTrackVisuals"), XXO("Set Track Visuals..."),
          CommandDispatch::OnAudacityCommand, AudioIONotBusyFlag() )
-   )
+   ),
+   wxT("Optional/Extra/Part2/Scriptables1")
 };
 
 AttachedItem sAttachment2{
-   wxT("Optional/Extra/Part2/Scriptables2"),
    // Note that the PLUGIN_SYMBOL must have a space between words,
    // whereas the short-form used here must not.
    // (So if you did write "Compare Audio" for the PLUGIN_SYMBOL name, then
    // you would have to use "CompareAudio" here.)
    Command( wxT("SetTrack"), XXO("Set Track..."),
-      CommandDispatch::OnAudacityCommand, AudioIONotBusyFlag() )
+      CommandDispatch::OnAudacityCommand, AudioIONotBusyFlag() ),
+   wxT("Optional/Extra/Part2/Scriptables2")
 };
 }
