@@ -15,16 +15,16 @@
 
 
 #include "DtmfGen.h"
+#include "EffectEditor.h"
 #include "LoadEffects.h"
 
-#include <wx/intl.h>
 #include <wx/slider.h>
 #include <wx/valgen.h>
 #include <wx/valtext.h>
 #include <wx/stattext.h>
 
 #include "Prefs.h"
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 #include "../widgets/NumericTextCtrl.h"
 #include "../widgets/valnum.h"
 
@@ -166,7 +166,7 @@ bool EffectDtmf::Instance::ProcessInitialize(
    auto &dtmfSettings = GetSettings(settings);
    if (dtmfSettings.dtmfNTones == 0) {   // Bail if no DTFM sequence.
       // TODO:  don't use mProcessor for this
-      mProcessor.MessageBox(
+      EffectUIServices::DoMessageBox(mProcessor,
                XO("DTMF sequence empty.\nCheck ALL settings for this effect."),
          wxICON_ERROR );
 
@@ -259,7 +259,7 @@ size_t EffectDtmf::Instance::ProcessBlock(EffectSettings &settings,
 
          // the statement takes care of extracting one sample from the diff bin and
          // adding it into the current block until depletion
-         numRemaining += (diff-- > 0 ? 1 : 0);         
+         numRemaining += (diff-- > 0 ? 1 : 0);
       }
 
       const auto len = limitSampleBufferSize( size, numRemaining );
@@ -289,18 +289,16 @@ size_t EffectDtmf::Instance::ProcessBlock(EffectSettings &settings,
 }
 
 // Event handler object
-struct EffectDtmf::Validator
-   : EffectUIValidator
+struct EffectDtmf::Editor
+   : EffectEditor
 {
-   Validator(EffectUIClientInterface &effect,
+   Editor(const EffectUIServices &effect,
       EffectSettingsAccess &access, const DtmfSettings &settings)
-      : EffectUIValidator{effect, access}
+      : EffectEditor{effect, access}
       // Copy settings
       , mSettings{settings}
    {}
-   virtual ~Validator() = default;
-
-   Effect &GetEffect() const { return static_cast<Effect&>(mEffect); }
+   virtual ~Editor() = default;
 
    bool ValidateUI() override;
    bool UpdateUI() override;
@@ -323,7 +321,7 @@ struct EffectDtmf::Validator
    wxStaticText *mDtmfDutyT;
 };
 
-void EffectDtmf::Validator::PopulateOrExchange(ShuttleGui & S,
+void EffectDtmf::Editor::PopulateOrExchange(ShuttleGui & S,
    const EffectSettings &settings, double projectRate)
 {
    // Reference to our copy of this effect's special settings
@@ -352,7 +350,7 @@ void EffectDtmf::Validator::PopulateOrExchange(ShuttleGui & S,
             return vldDtmf;
          })
          .AddTextBox(XXO("DTMF &sequence:"), wxT(""), 10);
-      BindTo(*mDtmfSequenceT, wxEVT_TEXT, &Validator::OnSequence);
+      BindTo(*mDtmfSequenceT, wxEVT_TEXT, &Editor::OnSequence);
 
       // A control with no event handler but the validator causes updates
       // when TransferData functions are called
@@ -365,16 +363,16 @@ void EffectDtmf::Validator::PopulateOrExchange(ShuttleGui & S,
       S.AddPrompt(XXO("&Duration:"));
       auto &extra = settings.extra;
       mDtmfDurationT = safenew
-         NumericTextCtrl(S.GetParent(), wxID_ANY,
-                         NumericConverter::TIME,
+         NumericTextCtrl(FormatterContext::SampleRateContext(projectRate),
+                         S.GetParent(), wxID_ANY,
+                         NumericConverterType_TIME(),
                          extra.GetDurationFormat(),
                          extra.GetDuration(),
-                         projectRate,
                          NumericTextCtrl::Options{}
                             .AutoPos(true));
       S.Name(XO("Duration"))
          .AddWindow(mDtmfDurationT);
-      BindTo(*mDtmfDurationT, wxEVT_TEXT, &Validator::OnDuration);
+      BindTo(*mDtmfDurationT, wxEVT_TEXT, &Editor::OnDuration);
 
       S.AddFixedText(XO("&Tone/silence ratio:"), false);
       mDtmfDutyCycleS =
@@ -385,7 +383,7 @@ void EffectDtmf::Validator::PopulateOrExchange(ShuttleGui & S,
                      dtmfSettings.dtmfDutyCycle * DutyCycle.scale,
                      DutyCycle.max * DutyCycle.scale,
                      DutyCycle.min * DutyCycle.scale);
-      BindTo(*mDtmfDutyCycleS, wxEVT_SLIDER, &Validator::OnDutyCycle);
+      BindTo(*mDtmfDutyCycleS, wxEVT_SLIDER, &Editor::OnDutyCycle);
    }
    S.EndMultiColumn();
 
@@ -395,7 +393,7 @@ void EffectDtmf::Validator::PopulateOrExchange(ShuttleGui & S,
       mDtmfDutyT =
       S.AddVariableText(XO("%.1f %%")
          .Format( dtmfSettings.dtmfDutyCycle ), false);
-      
+
       S.AddFixedText(XO("Tone duration:"), false);
       mDtmfSilenceT =
       /* i18n-hint milliseconds */
@@ -413,13 +411,13 @@ void EffectDtmf::Validator::PopulateOrExchange(ShuttleGui & S,
 
 // Effect implementation
 
-std::unique_ptr<EffectUIValidator> EffectDtmf::PopulateOrExchange(
+std::unique_ptr<EffectEditor> EffectDtmf::MakeEditor(
    ShuttleGui & S, EffectInstance &, EffectSettingsAccess &access,
-   const EffectOutputs *)
+   const EffectOutputs *) const
 {
    auto &settings = access.Get();
    auto &dtmfSettings = GetSettings(settings);
-   auto result = std::make_unique<Validator>(*this, access, dtmfSettings);
+   auto result = std::make_unique<Editor>(*this, access, dtmfSettings);
    result->PopulateOrExchange(S, settings, mProjectRate);
    return result;
 }
@@ -431,7 +429,7 @@ std::shared_ptr<EffectInstance> EffectDtmf::MakeInstance() const
    return std::make_shared<Instance>(*this, mT0);
 }
 
-bool EffectDtmf::Validator::UpdateUI()
+bool EffectDtmf::Editor::UpdateUI()
 {
    const auto &settings = mAccess.Get();
    auto &dtmfSettings = mSettings;
@@ -448,7 +446,7 @@ bool EffectDtmf::Validator::UpdateUI()
    return true;
 }
 
-bool EffectDtmf::Validator::ValidateUI()
+bool EffectDtmf::Editor::ValidateUI()
 {
    mAccess.ModifySettings([this](EffectSettings &settings){
       auto &dtmfSettings = mSettings;
@@ -643,7 +641,7 @@ bool EffectDtmf::MakeDtmfTone(float *buffer, size_t len, float fs, wxChar tone,
    return true;
 }
 
-void EffectDtmf::Validator::DoUpdateUI()
+void EffectDtmf::Editor::DoUpdateUI()
 {
    // Update some texts in response to controls
    auto &dtmfSettings = mSettings;
@@ -661,7 +659,7 @@ void EffectDtmf::Validator::DoUpdateUI()
    mDtmfToneT->SetName(mDtmfToneT->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
 }
 
-void EffectDtmf::Validator::OnSequence(wxCommandEvent & WXUNUSED(evt))
+void EffectDtmf::Editor::OnSequence(wxCommandEvent & WXUNUSED(evt))
 {
    mAccess.ModifySettings([this](EffectSettings &settings){
       auto &dtmfSettings = mSettings;
@@ -672,11 +670,10 @@ void EffectDtmf::Validator::OnSequence(wxCommandEvent & WXUNUSED(evt))
    DoUpdateUI();
 }
 
-void EffectDtmf::Validator::OnDuration(wxCommandEvent & WXUNUSED(evt))
+void EffectDtmf::Editor::OnDuration(wxCommandEvent & WXUNUSED(evt))
 {
    mAccess.ModifySettings([this](EffectSettings &settings){
       auto &dtmfSettings = mSettings;
-      auto &effect = GetEffect();
       settings.extra.SetDuration(mDtmfDurationT->GetValue());
       dtmfSettings.Recalculate(settings);
       return nullptr;
@@ -684,7 +681,7 @@ void EffectDtmf::Validator::OnDuration(wxCommandEvent & WXUNUSED(evt))
    DoUpdateUI();
 }
 
-void EffectDtmf::Validator::OnDutyCycle(wxCommandEvent & evt)
+void EffectDtmf::Editor::OnDutyCycle(wxCommandEvent & evt)
 {
    mAccess.ModifySettings([this, &evt](EffectSettings &settings){
       auto &dtmfSettings = mSettings;
