@@ -102,27 +102,14 @@ bool EffectStereoToMono::Process(EffectInstance &, EffectSettings &)
 
    // Process each stereo track
    sampleCount curTime = 0;
-   bool refreshIter = false;
 
    mProgress->SetMessage(XO("Mixing down to mono"));
 
-   // Don't use range-for, because iterators may be invalidated by erasure from
-   // the track list
-   while (trackRange.first != trackRange.second) {
-      auto track = *trackRange.first;
+   for (const auto track : trackRange) {
       if (track->Channels().size() > 1) {
          if (!ProcessOne(outputs.Get(), curTime, totalTime, *track))
             break;
-         // The right channel has been deleted, so we must restart from the beginning
-         refreshIter = true;
       }
-
-      if (refreshIter) {
-         trackRange = outputs.Get().Selected<WaveTrack>();
-         refreshIter = false;
-      }
-      else
-         ++trackRange.first;
    }
 
    if (bGoodResult)
@@ -145,21 +132,17 @@ bool EffectStereoToMono::ProcessOne(TrackList &outputs,
    tracks.emplace_back(
       track.SharedPointer<const SampleTrack>(), GetEffectStages(track));
 
-   Mixer mixer(move(tracks),
-      true,                // Throw to abort mix-and-render if read fails:
-      Mixer::WarpOptions{ inputTracks()->GetOwner() },
-      start,
-      end,
-      1,
+   Mixer mixer(
+      move(tracks), std::nullopt,
+      true, // Throw to abort mix-and-render if read fails:
+      Mixer::WarpOptions { inputTracks()->GetOwner() }, start, end, 1,
       idealBlockLen,
-      false,               // Not interleaved
-      track.GetRate(),
-      floatSample);
+      false, // Not interleaved
+      track.GetRate(), floatSample);
 
-   // Always make mono output; don't use WideEmptyCopy
-   auto outTrack = track.EmptyCopy();
-   auto tempList = TrackList::Temporary(nullptr, outTrack, nullptr);
-   assert(outTrack->IsLeader());
+   // Always make mono output; don't use EmptyCopy
+   auto outTrack = track.EmptyCopy(1);
+   auto tempList = TrackList::Temporary(nullptr, outTrack);
    outTrack->ConvertToSampleFormat(floatSample);
 
    double denominator = track.GetChannelGain(0) + track.GetChannelGain(1);
@@ -172,7 +155,8 @@ bool EffectStereoToMono::ProcessOne(TrackList &outputs,
       // (for example), and no gains or envelopes, still there should be
       // dithering because of the averaging above, which may introduce samples
       // lying between the quantization levels.  So use widestSampleFormat.
-      outTrack->Append(buffer, floatSample, blockLen, 1, widestSampleFormat);
+      outTrack->Append(0,
+         buffer, floatSample, blockLen, 1, widestSampleFormat);
 
       curTime += blockLen;
       if (TotalProgress(curTime.as_double() / totalTime.as_double()))
@@ -180,11 +164,8 @@ bool EffectStereoToMono::ProcessOne(TrackList &outputs,
    }
    outTrack->Flush();
 
-   const auto unlinkedTracks = outputs.UnlinkChannels(track);
-   assert(unlinkedTracks.size() == 2);
-   outputs.Remove(*unlinkedTracks[1]);
-
    track.Clear(start, end);
+   track.MakeMono();
    track.Paste(start, *outTrack);
    RealtimeEffectList::Get(track).Clear();
 
