@@ -44,13 +44,13 @@ const static wxChar *kShippedEffects[] =
    wxT("rissetdrum.ny"),
    wxT("sample-data-export.ny"),
    wxT("sample-data-import.ny"),
+   wxT("ShelfFilter.ny"),
    wxT("spectral-delete.ny"),
    wxT("SpectralEditMulti.ny"),
    wxT("SpectralEditParametricEQ.ny"),
    wxT("SpectralEditShelves.ny"),
    wxT("StudioFadeOut.ny"),
    wxT("tremolo.ny"),
-   wxT("vocalrediso.ny"),
    wxT("vocoder.ny"),
 };
 
